import matplotlib.pyplot as plt
import numpy as np
import pandas as pd


def omega(k, gamma, M_s, A, T, B_0=0, kind="BVW", n=0, mu_0=4*np.pi*1e-7):

    omega_M = gamma * mu_0 * M_s
    omega_0 = gamma * B_0
    Lambda = np.sqrt(2*A / (mu_0 *M_s**2))


    k_n = np.sqrt(k**2 + (n*np.pi/T)**2)

    rat = k**2 / k_n**2
    kron_n = (n == 0)
    P_nn = rat*(1 - (2/(1+kron_n))*rat*(1-(-1)**n *np.exp(-k*T))/(k*T))
    if kind == "BVW":
        return np.sqrt(
            (omega_0 + omega_M* Lambda**2 * k_n**2) *\
                (omega_0 + omega_M* Lambda**2 *k_n**2 + omega_M*(1-P_nn))
        )
    if kind == "SW":
        return np.sqrt(
            (omega_0 + omega_M * Lambda**2 *k_n**2 + omega_M*P_nn) *\
                (omega_0 + omega_M* Lambda**2 *k_n**2 + omega_M*(1-P_nn))
        )
    else:
        print("No valid spin wave kind specified in dispersion. "
              "Please use BVW or SW.")
        return k*0

def omega_gus(k, gamma, M_s, A, T, B_0=0, kind="BVW", b=-1, ny=0, mu_0=4*np.pi*1e-7):

    omega_M = gamma * mu_0 * M_s
    omega_0 = gamma * B_0

    Lambda = np.sqrt(2*A / (mu_0 *M_s**2))

    if b > 0:
        ## TODO Check if this is correct
        # employ Guslienko boundary conditions for finite stripe width
        p = T / b
        d = 2*np.pi / (p*(1 + 2 * np.log(1/p)))
        beff = b*d / (d-2)
        ky = (ny + 1)* np.pi / beff
        k = np.sqrt(k**2 + ky**2)

    P = ( 1 - np.exp(-k*T))/(k*T)

    if kind == "BVW":
        return np.sqrt(
            (omega_0 + omega_M* Lambda**2 * k**2) *\
                (omega_0 + omega_M* Lambda**2 *k**2 + omega_M*P)
        )
    if kind == "SW":
        return np.sqrt(
            (omega_0 + omega_M * Lambda**2 * k**2) *\
                (omega_0 + omega_M* Lambda**2 *k**2 + omega_M) + omega_M**2 / 4 *\
                (1-np.exp(-2*k*T))
        )
    else:
        print("No valid spin wave kind specified in dispersion. "
              "Please use BVW or SW.")
        return k*0





def main():
    gamma = 1.760859644e11 # 1/Ts # *
    A = 11e-12 # J/m
    M_s = 800e3 # A/m
    mu_0 = 4*np.pi*1e-7 # N / A**2
    B_0 = 20e-3#200e-3 # T
    T = 80e-9 # nm"""
    b = 1e-6
    omega_0 = gamma * B_0


    k = np.linspace(0.1,40*1e6,1000)

    d_ = [10,20,50,75,100]
    #d_ = [10]
    fig, (ax1) = plt.subplots(1)
    numberofmodes = 3
    #plt.axhline(y=omega_0/(2*np.pi)*1e-9, c="k", ls=":")
    for d in d_:
        d *= 1e-9
        df = pd.DataFrame({"k (rad/m)" : k})
        for n in np.arange(0, numberofmodes):
            f_BVM = omega(k, gamma, M_s, A, d, B_0, kind="BVW", n=n)/(2*np.pi)*1e-9
            f_SW = omega(k, gamma, M_s, A, d, B_0, kind="SW", n=n)/(2*np.pi)*1e-9
            df[f"f_BVW_{n} (GHz)"] = f_BVM
            df[f"f_SW_{n} (GHz)"] = f_SW
            #ax1.plot(k*1e-6,f_BVM,c="C"+str(n), ls="-.")
            #ax1.plot(k*1e-6,f_SW,c="C"+str(n), ls="-")
            df.to_csv(f"dispersion_theory_d{d*1e9}nm.csv")




    #ax1.set_xlabel("Wave vector, $k$ (rad/µm)")
    #ax1.set_ylabel("Frequency, $\omega/2\pi$ (GHz)")


    #k = np.linspace(0.1,30*1e6,1000)
    #ax1.plot(k*1e-6,omega(k, gamma, M_s, A, T, B_0, kind="BVW", n=0)/                    (2*np.pi)*1e-9,c="C0", ls="-.")
    #ax1.plot(k*1e-6,omega(k, gamma, M_s, A, T, B_0, kind="SW", n=0)/(2*np.pi)*1e-9,c="C0", ls="-")
    #plt.show()

if __name__ == "__main__":
    main()
